/*
 * xrick/src/sysvid.c
 *
 * This source code file is part of the xrick distribution, and as such is
 * copyrighted. Please read the README file in the root directory of the
 * distribution for information about copyrights, trademarks, license,
 * disclaimers, etc.
 */

#include "gp32.h"

#include "system.h"
#include "game.h"
#include "img.h"
#include "debug.h"

U8 *sysvid_fb; /* frame buffer */
rect_t SCREENRECT = {0, 0, SYSVID_WIDTH, SYSVID_HEIGHT, NULL}; /* whole fb */

int			vid_page;
int			drw_page;
GPDRAWSURFACE	gpDraw[2];
GP_HPALETTE		pal_cur, pal_old;
GP_PALETTEENTRY	palette[256];

static U32 videoFlags;

static U8 zoom = 1; /* actual zoom level */
static U8 szoom = 0;  /* saved zoom level */
static U8 fszoom = 0;  /* fullscreen zoom level */

/*
 * color tables
 */

static U8 RED[] = { 0x00, 0xd8, 0xb0, 0xf8,
                    0x20, 0x00, 0x00, 0x20,
                    0x48, 0x48, 0x90, 0xd8,
                    0x48, 0x68, 0x90, 0xb0,
                    /* cheat colors */
                    0x50, 0xe0, 0xc8, 0xf8,
                    0x68, 0x50, 0x50, 0x68,
                    0x80, 0x80, 0xb0, 0xe0,
                    0x80, 0x98, 0xb0, 0xc8
};
static U8 GREEN[] = { 0x00, 0x00, 0x6c, 0x90,
                      0x24, 0x48, 0x6c, 0x48,
                      0x6c, 0x24, 0x48, 0x6c,
                      0x48, 0x6c, 0x90, 0xb4,
		      /* cheat colors */
                      0x54, 0x54, 0x9c, 0xb4,
                      0x6c, 0x84, 0x9c, 0x84,
                      0x9c, 0x6c, 0x84, 0x9c,
                      0x84, 0x9c, 0xb4, 0xcc
};
static U8 BLUE[] = { 0x00, 0x00, 0x68, 0x68,
                     0x20, 0xb0, 0xd8, 0x00,
                     0x20, 0x00, 0x00, 0x00,
                     0x48, 0x68, 0x90, 0xb0,
		     /* cheat colors */
                     0x50, 0x50, 0x98, 0x98,
                     0x68, 0xc8, 0xe0, 0x50,
                     0x68, 0x50, 0x50, 0x50,
                     0x80, 0x98, 0xb0, 0xc8};

/*
 * Initialize screen
 */

void sysvid_setPalette(img_color_t *pal, U16 n)
{
	U16 i;
	U16 r;
	U16 g;
	U16 b;

	for(i=0; i<n; i++){
		r = (pal[i].r >> 3);
		g = (pal[i].g >> 3);
		b = (pal[i].b >> 3);
#if 1
		// make brighter
		r += 8;
		g += 8;
		b += 8;
		if(r > 0x1F) r = 0x1F;
		if(g > 0x1F) g = 0x1F;
		if(b > 0x1F) b = 0x1F;
#endif
		r &= 0x1F; g &=0x1F; b &= 0x1F;
		palette[i] = (r<<11) | (g<<6) | (b<<1) | 0;
	}

	pal_cur = GpPaletteCreate(256, palette);	
	GpPaletteSelect(pal_cur);
	GpPaletteRealize();
}


void sysvid_restorePalette()
{
}

void sysvid_setGamePalette()
{
  U8 i;
  img_color_t pal[256];

  for (i = 0; i < 32; ++i) {
    pal[i].r = RED[i];
    pal[i].g = GREEN[i];
    pal[i].b = BLUE[i];
  }
  sysvid_setPalette(pal, 32);
}

/*
 * Initialise video
 */
void sysvid_init(void)
{
	videoFlags = 0;
	zoom = 1;
	szoom = 1;
	fszoom = 1;

	GpGraphicModeSet(8, NULL);

	GpLcdSurfaceGet( &gpDraw[0], 0);
	GpLcdSurfaceGet( &gpDraw[1], 1);

	vid_page = 0;
	GpSurfaceSet( &gpDraw[vid_page] );

	sysvid_fb = gm_malloc(SYSVID_WIDTH * SYSVID_HEIGHT);
	if (!sysvid_fb){
    		sys_panic("xrick/video: sysvid_fb malloc failed\n");
	}

	// Turn on screen
	if( !(GPC_LCD_ON_BIT & GpLcdStatusGet()))
		GpLcdEnable();

	// Clear the screen
	GpRectFill(NULL, &gpDraw[vid_page], 0, 0, 320, 240, 0x00);	

}

/*
 * Shutdown video
 */
void sysvid_shutdown(void)
{
  gm_free(sysvid_fb);
  sysvid_fb = NULL;
}

/*
 * Update screen
 * NOTE errors processing ?
 */
void sysvid_update(rect_t *rects)
{
  U16 x, y, xz, yz;
  U8 *p, *q, *p0, *q0;

  if (rects == NULL)
    return;

  while (rects) {
    p0 = sysvid_fb;
    p0 += rects->x + rects->y * SYSVID_WIDTH;
  
    q0 = (U8 *)gpDraw[vid_page].o_buffer;
    q0 += ((rects->x*240) + (219-rects->y));

    for (y = rects->y; y < rects->y + rects->height; y++) {
      for (yz = 0; yz < zoom; yz++) {
	p = p0;
	q = q0;
	for (x = rects->x; x < rects->x + rects->width; x++) {
	  *q = *(p++);
	  q+=240;
	}
	q0 --;
      }
      p0 += SYSVID_WIDTH;
    }
    rects = rects->next;
  }
}


/*
 * Clear screen
 * (077C)
 */
void sysvid_clear(void)
{
  gm_memset(sysvid_fb, 0, SYSVID_WIDTH * SYSVID_HEIGHT);
}


/*
 * Zoom
 */
void sysvid_zoom(S8 z)
{
}

/*
 * Toggle fullscreen
 */
void sysvid_toggleFullscreen(void)
{
}

/* eof */



